@@ -1,5 +1,5 @@
 #    https://launchpad.net/wxbanker
-#    persistentstore.py: Copyright 2007-2009 Mike Rooney <mrooney@ubuntu.com>
+#    persistentstore.py: Copyright 2007-2010 Mike Rooney <mrooney@ubuntu.com>
 #
 #    This file is part of wxBanker.
 #
@@ -17,12 +17,12 @@
 #    along with wxBanker.  If not, see <http://www.gnu.org/licenses/>.
 
 """
-Table: accounts                                v2                 v3
-+---------------------------------------------------------------+---------------+
-| id INTEGER PRIMARY KEY | name VARCHAR(255) | currency INTEGER | balance FLOAT |
-|------------------------+-------------------|------------------|---------------|
-| 1                      | "My Account"      | 0                | 0             |
-+---------------------------------------------------------------+---------------+
+Table: accounts                                v2                 v3              v10
++---------------------------------------------------------------+---------------+-----------------+
+| id INTEGER PRIMARY KEY | name VARCHAR(255) | currency INTEGER | balance FLOAT | mintId INTEGER  |
+|------------------------+-------------------+------------------+---------------+-----------------|
+| 1                      | "My Account"      | 0                | 0             | 123456          |
++---------------------------------------------------------------+---------------+-----------------+
 
 Table: transactions                                                                                       v4
 +-------------------------------------------------------------------------------------------------------+----------------+
@@ -53,7 +53,7 @@ class PersistentStore:
     """
     def __init__(self, path, autoSave=True):
         self.Subscriptions = []
-        self.Version = 9
+        self.Version = 11
         self.Path = path
         self.AutoSave = False
         self.Dirty = False
@@ -110,9 +110,7 @@ class PersistentStore:
     def GetModel(self, useCached=True):
         if self.cachedModel is None or not useCached:
             debug.debug('Creating model...')
-            accounts = self.getAccounts()
-            accountList = AccountList(self, accounts)
-            self.cachedModel = BankModel(self, accountList)
+            self.cachedModel = BankModel(self)
 
         return self.cachedModel
 
@@ -124,15 +122,11 @@ class PersistentStore:
             raise Exception("Currency code must be int and >= 0")
 
         cursor = self.dbconn.cursor()
-        cursor.execute('INSERT INTO accounts VALUES (null, ?, ?, ?)', (accountName, currency, 0.0))
+        cursor.execute('INSERT INTO accounts (name, currency) VALUES (?, ?)', (accountName, currency))
         ID = cursor.lastrowid
         self.commitIfAppropriate()
 
         account = Account(self, ID, accountName, currency)
-
-        # Ensure there are no orphaned transactions, for accounts removed before #249954 was fixed.
-        self.clearAccountTransactions(account)
-
         return account
 
     def RemoveAccount(self, account):
@@ -154,12 +148,16 @@ class PersistentStore:
         return transaction
 
     def RemoveTransaction(self, transaction):
-        result = self.dbconn.cursor().execute('DELETE FROM transactions WHERE id=?', (transaction.ID,)).fetchone()
+        ID = transaction.ID
+        result = self.dbconn.cursor().execute('DELETE FROM transactions WHERE id=?', (ID,)).fetchone()
         self.commitIfAppropriate()
-        # The result doesn't appear to be useful here, it is None regardless of whether the DELETE matched anything
-        # the controller already checks for existence of the ID though, so if this doesn't raise an exception, theoretically
-        # everything is fine. So just return True, as there we no errors that we are aware of.
+        # The result doesn't appear to be useful here, it is None regardless of whether the DELETE matched anything.
         return True
+    
+    def RemoveRecurringTransaction(self, recurring):
+        ID = recurring.ID
+        result = self.dbconn.cursor().execute('DELETE FROM recurring_transactions WHERE id=?', (ID,))
+        self.commitIfAppropriate()
 
     def Save(self):
         import time; t = time.time()
@@ -278,8 +276,28 @@ class PersistentStore:
         elif fromVer == 8:
             # Add the LastAccountId meta key.
             cursor.execute('INSERT INTO meta VALUES (null, ?, ?)', ('LastAccountId', None))
+        elif fromVer == 9:
+            # Mint integration
+            cursor.execute('INSERT INTO meta VALUES (null, ?, ?)', ('MintEnabled', False))
+            cursor.execute('ALTER TABLE accounts ADD mintId INTEGER')
+        elif fromVer == 10:
+            # The obvious index to create, had I known about indexes previously.
+            # It is nice to use IF NOT EXISTS here as some devs and branch users might have it outside of an upgrade.
+            cursor.execute('CREATE INDEX IF NOT EXISTS transactions_accountId_idx ON transactions(accountId)')
+            # Due to bug LP #605591 there can be orphaned transactions which can cause errors if linked.
+            # This is tested by testOrphanedTransactionsAreDeleted (dbupgradetests.DBUpgradeTest)
+            # Also takes care of LP #249954 without the explicit need to defensively remove on any account creation.
+            self.cleanOrphanedTransactions()
         else:
             raise Exception("Cannot upgrade database from version %i"%fromVer)
+        
+        """
+        # Tagging infrastructure (currently unused due to speed of parsing on startup).
+        cursor.execute('CREATE TABLE tags (id INTEGER PRIMARY KEY, name VARCHAR(255))')
+        cursor.execute('CREATE TABLE transactions_tags_link (id INTEGER PRIMARY KEY, transactionId INTEGER, tagId INTEGER)')
+        cursor.execute('CREATE INDEX transactions_tags_transactionId_idx ON transactions_tags_link(transactionId)')
+        cursor.execute('CREATE INDEX transactions_tags_tagId_idx ON transactions_tags_link(tagId)')
+        """
 
         metaVer = fromVer + 1
         cursor.execute('UPDATE meta SET value=? WHERE name=?', (metaVer, "VERSION"))
@@ -308,8 +326,8 @@ class PersistentStore:
         return result
 
     def result2account(self, result):
-        ID, name, currency, balance = result
-        return Account(self, ID, name, currency, balance)
+        ID, name, currency, balance, mintId = result
+        return Account(self, ID, name, currency, balance, mintId)
     
     def result2recurringtransaction(self, result, parentAccount, allAccounts):
         rId, accountId, amount, description, date, repeatType, repeatEvery, repeatOn, endDate, sourceId, lastTransacted = result
@@ -324,9 +342,12 @@ class PersistentStore:
 
         return RecurringTransaction(rId, parentAccount, amount, description, date, repeatType, repeatEvery, repeatOn, endDate, sourceAccount, lastTransacted)
 
-    def getAccounts(self):
+    def getAccountRows(self):
+        return self.dbconn.cursor().execute("SELECT * FROM accounts").fetchall()
+        
+    def GetAccounts(self):
         # Fetch all the accounts.
-        accounts = [self.result2account(result) for result in self.dbconn.cursor().execute("SELECT * FROM accounts").fetchall()]
+        accounts = [self.result2account(result) for result in self.getAccountRows()]
         # Add any recurring transactions that exist for each.
         recurrings = self.getRecurringTransactions()
         for recurring in recurrings:
@@ -339,10 +360,25 @@ class PersistentStore:
     
     def getRecurringTransactions(self):
         return self.dbconn.cursor().execute('SELECT * FROM recurring_transactions').fetchall()
-
-    def clearAccountTransactions(self, account):
-        self.dbconn.cursor().execute('DELETE FROM transactions WHERE accountId=?', (account.ID,))
-        self.commitIfAppropriate()
+        
+    def cleanOrphanedTransactions(self):
+        # Grab all the accounts that currently exist, to check against.
+        accountIDs = [row[0] for row in self.getAccountRows()]
+        # We'll keep a list of deceased accounts to clean, so we aren't doing a DELETE per orphan.
+        deceasedAccounts = set()
+        
+        # Iterate over the transactions, treating the cursor as an iterator instead of fetchall() in case there are a lot.
+        for transactionRow in self.dbconn.cursor().execute('SELECT * FROM transactions'):
+            accountID = transactionRow[1]
+            # If the account ID isn't in the current accounts, it must not exist and this transaction is an orphan, so note the parent.
+            if accountID not in accountIDs:
+                deceasedAccounts.add(accountID)
+                
+        # Now iterate over the deceased accounts of which to remove orphans.
+        for accountID in deceasedAccounts:
+            self.dbconn.cursor().execute('DELETE FROM transactions WHERE accountId=?', (accountID,))
+        
+        self.commitIfAppropriate()        
 
     def result2transaction(self, result, parentObj, linkedTransaction=None, recurringCache=None):
         tid, pid, amount, description, date, linkId, recurringId = result
@@ -361,7 +397,7 @@ class PersistentStore:
                 try:
                     link, linkAccount = self.getTransactionAndParentById(linkId, parentObj, t)
                 except MissingLinkException:
-                    # The link is gone, it's Account was likely deleted before LP: #514183 was fixed. Remove it.
+                    # The link is gone, it's Account was likely deleted before LP: #514183/605591 was fixed. Remove it.
                     t.LinkedTransaction = None
                 else:
                     # If the link parent hasn't loaded its transactions yet, put this in its pre list so this
