@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #    https://launchpad.net/wxbanker
-#    modeltests.py: Copyright 2007-2009 Mike Rooney <mrooney@ubuntu.com>
+#    modeltests.py: Copyright 2007-2010 Mike Rooney <mrooney@ubuntu.com>
 #
 #    This file is part of wxBanker.
 #
@@ -295,7 +295,32 @@ class ModelTests(testbase.TestCaseWithController):
         result = a.RemoveTransaction(ta)
         self.assertEqual(result, [b], result[0].Name)
         
-    def testCanMoveTransfer(self):
+    def testCanMoveTransferSource(self):
+        model = self.Controller.Model
+        a = model.CreateAccount("A")
+        b = model.CreateAccount("B")
+        
+        atrans, btrans = a.AddTransaction(1, source=b)
+        self.assertEqual(len(model.GetTransactions()), 2)
+        self.assertEqual(model.Balance, 0)
+        self.assertEqual(atrans.Description, "Transfer from B")
+        self.assertEqual(btrans.Description, "Transfer to A")
+        
+        c = model.CreateAccount("C")
+        b.MoveTransaction(btrans, c)
+        
+        self.assertEqual(b.Transactions, [])
+        self.assertEqual(len(a.Transactions), 1)
+        self.assertEqual(len(c.Transactions), 1)
+        
+        atrans = a.Transactions[0]
+        ctrans = c.Transactions[0]
+        self.assertEqual(atrans.LinkedTransaction, ctrans)
+        self.assertEqual(ctrans.LinkedTransaction, atrans)
+        self.assertEqual(atrans.Description, "Transfer from C")
+        self.assertEqual(ctrans.Description, "Transfer to A")
+        
+    def testCanMoveTransferDestination(self):
         model = self.Controller.Model
         a = model.CreateAccount("A")
         b = model.CreateAccount("B")
@@ -424,10 +449,10 @@ class ModelTests(testbase.TestCaseWithController):
         t.Date = None
         self.assertEqual(t.Date, datetime.date.today())
         
-    def testDeletingAccountDoesNotSiblingLinkedTransfers(self):
+    def testDeletingAccountDoesNotDeleteSiblingLinkedTransfers(self):
         """If you close (delete) an account, it is still true that the transfers occurred."""
         a, b, atrans, btrans = self.createLinkedTransfers()
-        model = self.Controller.Model
+        model = self.Model
         
         self.assertTrue(atrans in a.Transactions)
         self.assertTrue(btrans in b.Transactions)
@@ -439,6 +464,84 @@ class ModelTests(testbase.TestCaseWithController):
         self.assertTrue(atrans in a.Transactions)
         self.assertEqual(atrans.LinkedTransaction, None)
         
+    def testDefaultMintIntegrationIsFalse(self):
+        self.assertEqual(self.Model.MintEnabled, False)
+        
+    def testAccountMintSync(self):
+        model = self.Model
+        a = model.CreateAccount("Foo")
+        
+        self.assertFalse(a.IsMintEnabled())
+        self.assertRaises(bankexceptions.MintIntegrationException, a.IsInSync)
+
+        a.MintId = 1218040
+        self.assertEquals(1218040, a.GetMintId())
+        self.assertTrue(a.IsMintEnabled())
+
+        # Put the fixture cached summary in, so we can test.
+        from wxbanker.mint.api import MintConnection
+        MintConnection()._CachedSummary = open(testbase.fixturefile("mint_index.html")).read()
+        
+        self.assertFalse(a.IsInSync())
+
+        # Add the balance and we should be in sync.
+        a.AddTransaction(-4277.24)
+        self.assertTrue(a.IsInSync())
+        
+        self.assertEqual(a.GetSyncString(), "PayPal PayPal Balance: -$4,277.24")
+
+        del MintConnection()._CachedSummary
+        
+    def testAccountMintSyncWithFutureDates(self):
+        model = self.Model
+        a = model.CreateAccount("Foo")
+        a.MintId = 1218040
+
+        # Put the fixture cached summary in, so we can test.
+        from wxbanker.mint.api import MintConnection
+        MintConnection()._CachedSummary = open(testbase.fixturefile("mint_index.html")).read()
+        
+        # Add the balance and we should be in sync.
+        a.AddTransaction(-4277.24)
+        self.assertTrue(a.IsInSync())
+  
+        # Add a transaction today, we should be out of sync.
+        t = a.AddTransaction(1)
+        self.assertFalse(a.IsInSync())
+        
+        # Change the date to tomorrow, we should be back in sync as of today.
+        t.Date = tomorrow
+        self.assertTrue(a.IsInSync())
+        
+    def testAccountMintIdIsInt(self):
+        model = self.Model
+        a = model.CreateAccount("Foo")
+        
+        a.MintId = "12345"
+        self.assertEquals(a.MintId, 12345)
+        
+    def testAccountCurrentBalance(self):
+        model = self.Model
+        a = model.CreateAccount("Bar")
+        
+        self.assertEqual(a.Balance, 0)
+        self.assertEqual(a.CurrentBalance, 0)
+        
+        t1 = a.AddTransaction(1)
+        self.assertEqual(a.Balance, 1)
+        self.assertEqual(a.CurrentBalance, 1)
+ 
+        t2 = a.AddTransaction(1, date=tomorrow)
+        self.assertEqual(a.Balance, 2)
+        self.assertEqual(a.CurrentBalance, 1)
+        
+        # Make sure that it isn't just by transaction order but actually date.
+        t2.Date = today
+        t1.Date = tomorrow
+        self.assertEqual(a.Balance, 2)
+        self.assertEqual(a.CurrentBalance, 1)
+        
+ 
         
 if __name__ == "__main__":
     unittest.main()
